/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.core;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.util.Vector;

import org.ugasp.eventmodel.event.Event;

/**
 * ActorSession represent the link between a user (represented by instances of Session and Actor)
 * and an ApplicationInstance.
 * It receives and transmits all the events or messages in an ApplicationInstance, in game it is the
 * communication link  between the client and the game session server.
 * It also contains attributes used by data transmission protocol to store events until client has
 * acknowledged their reception.
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public interface ActorSession {

	
	/**
	 * Returns the ApplicationInstance ID linked.
	 * 
	 * @return the ApplicationInstance ID
	 */
	int getApplicationInstanceID();


	/**
	 * Returns the Actor ID of the user.
	 * 
	 * @return the Actor ID
	 */
	int getActorID();
	

	/**
	 * Returns the Session ID of the user
	 * 
	 * @return the Session ID
	 */
	int getSessionID();

	
	/**
	 * Returns the ActorSession ID.
	 * 
	 * @return the ActorSession ID.
	 */
	short getActorSessionID();

	
	/**
	 * Returns the current pseudo name of the user session.
	 * 
	 * @return the ActorSession ID
	 */
	String getPseudoName();
	
	
	/**
	 * Set the current pseudo name of the user session.
	 */
	void setPseudoName(String name);

	
	/**
	 * Determines if the pseudo has modified during the session.
	 */
	boolean isPseudoHasModified();
	
	
	/**
	 * Return the events stored since the last request from the actor.
	 * 
	 * @return the stack of events
	 */
	Vector getEvents();
	
	
	/**
	 * Put the event on the stack of the events must be read by the actor.
	 * 
	 * @param e the Event
	 */
	void raiseEvent(Event e);
	
	
	/**
	 * Get the ID of the last message sent to this ActorSession.
	 * @return	the ID of last message sent to client
	 */
	public byte getMessageSentID();
	
	
	/**
	 * Store the ID of the last message sent to this ActorSession.
	 * 
	 * @param	messageSentID theID of the last message sent
	 */
	public void setMessageSentID(byte messageSentID);
	
	
	/**
	 * Get the ID of the last message received from this ActorSession.
	 * 
	 * @return	the ID of last message received from client
	 */
	public byte getLastMessageReceivedID();
	
	
	/**
	 * Store the ID of the last received from  this ActorSession.
	 * 
	 * @param	messageSentID theID of the last message sent
	 */
	public void setLastMesageReceivedID(byte lastMessageReceivedID);
	
	
	/**
	 * Get the ByteArrayOutputStream of the send buffer designed to this ActorSession.
	 * The send buffer is used to send events designed to a client. It is reset (via its ByteArrayOutputStream)
	 * when the client has acknowledged the reception of those events.
	 * 
	 * @return	the ByteArrayOutputStream of the send buffer designed to client
	 */
	public ByteArrayOutputStream getSendBufferByteArrayOutputStream();
	
	
	/**
	 * Store the ByteArrayOutputStream of the send buffer designed to this ActorSession.
	 * The send buffer is used to send events designed to a client. It is reset (via its ByteArrayOutputStream)
	 * when the client has acknowledged the reception of those events.
	 * 
	 * @param	ByteArrayOutputStream of the send buffer designed to client
	 */
	public void setSendBufferByteArrayOutputStream(ByteArrayOutputStream sendBufferByteArrayOutputStream);
	
	/**
	 * Get the DataOutputStream of the send buffer designed to this ActorSession.
	 * The send buffer is used to send events designed to a client. It is reset (via its ByteArrayOutputStream) 
	 * when the client has acknowledged the reception of those events.
	 * 
	 * @return	the ByteArrayOutputStream of the send buffer designed to client
	 */
	public DataOutputStream getSendBufferDataOutputStream();
	
	
	/**
	 * Store the DataOutputStream of the send buffer designed to this ActorSession.
	 * The send buffer is used to send events designed to a client. It is reset (via its ByteArrayOutputStream)
	 * when the client has acknowledged the reception of those events.
	 * 
	 * @param	DataOutputStream of the send buffer designed to client
	 */
	public void setSendBufferDataOutputStream(DataOutputStream sendBufferDataOutputStream);

}
