/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.impl.core;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.util.Vector;

import org.ugasp.core.ActorSession;
import org.ugasp.core.Session;
import org.ugasp.eventmodel.event.Event;
import org.ugasp.logging.Logger;

/**
 * ActorSession represent the link between a user (represented by instances of Session and Actor)
 * and an ApplicationInstance.
 * It receives and transmits all the events or messages in an ApplicationInstance, in game it is the
 * communication link  between the client and the game session server.
 * It also contains attributes used by data transmission protocol to store events until client has
 * acknowledged their reception.
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public class ActorSessionImpl implements ActorSession {

	private int applicationInstanceID;
	private int actorID;
	private int sessionID;
	private short actorSessionID;
	private String pseudo;
	private Vector<Event> events;
	private byte messageSentID;
	private byte lastMessageReceivedID;
	private ByteArrayOutputStream sendBufferByteArrayOutputStream;
	private DataOutputStream sendBufferDataOutputStream;
	
	// storage of events must be send to the user represented by the actor session.
    private boolean pseudoModified;

	public ActorSessionImpl(int aIID, Session session){
		applicationInstanceID = aIID;
		actorID = session.getActorID();
		sessionID = session.getSessionID();
		actorSessionID = PlatformImpl.getPlatform().requestASID();
		PlatformImpl.getPlatform().log(Logger.INFO,ActorSessionImpl.class,"Actor with ID = "+actorID+" and sessionID = "+sessionID+" has been affected actorSessionID = "+actorSessionID);
		session.setActorSessionID(actorSessionID);
		events = new Vector<Event>();
		// init data transmission attributes
		initTransmissionsAttributes();
	}
	
	
	private void initTransmissionsAttributes(){
		sendBufferByteArrayOutputStream = new ByteArrayOutputStream();
		sendBufferDataOutputStream = new DataOutputStream(sendBufferByteArrayOutputStream);
		messageSentID = 0;
	}

	
	/**
	 * Returns the ApplicationInstance ID linked.
	 * 
	 * @return the ApplicationInstance ID
	 */
	public int getApplicationInstanceID() {
		return applicationInstanceID;
	}
	
	
	/**
	 * Returns the Actor ID of the user.
	 * 
	 * @return the Actor ID
	 */
	public int getActorID() {
		return actorID;
	}

	
	/**
	 * Returns the Session ID of the user
	 * 
	 * @return the Session ID
	 */
	public int getSessionID() {
		return sessionID;
	}
	
	
	/**
	 * Returns the ActorSession ID.
	 * 
	 * @return the ActorSession ID.
	 */
	public short getActorSessionID() {
		return actorSessionID;
	}
	
	
	/**
	 * Returns the current pseudo name of the user session.
	 * 
	 * @return the ActorSession ID
	 */
	public String getPseudoName() {
		return pseudo;
	}
	
	
	/**
	 * Set the current pseudo name of the user session.
	 */
	public void setPseudoName(String name) {
		pseudo = name;
		pseudoModified = true;
	}
	
	
	/**
	 * Tells if the pseudo has modified during the session.
	 */
	public boolean isPseudoHasModified() {
		return pseudoModified;
	}

	
	/**
	 * Return the events stored since the last request from the actor.
	 * 
	 * @return the stack of events
	 */
	public synchronized Vector<Event> getEvents(){
        Vector<Event> v = new Vector<Event>(events);
	    events.removeAllElements();
	    return purgeEvents(v);
	}
	
	
	/**
	 * Put the event on the stack of the events
	 * 
	 * @param e the Event
	 */
	public synchronized void raiseEvent(Event e){
		events.add(e);
	    PlatformImpl.getPlatform().log(Logger.DEBUG,ActorSessionImpl.class,"Event with code = "+e.getCode()+" raised to Actor Session with ID = "+actorSessionID);
	}
	

    /**
     * Purge the events.
     * 
     * @param v the copy of the events vector
     */
    private Vector<Event> purgeEvents(Vector<Event> v) {
       return v; 
    }


    /******************************************************************/
    /*************   Data Transmission Management   *******************/
    /******************************************************************/
    
	@Override
	public byte getLastMessageReceivedID() {
		return lastMessageReceivedID;
	}


	@Override
	public byte getMessageSentID() {
		return messageSentID;
	}


	@Override
	public ByteArrayOutputStream getSendBufferByteArrayOutputStream() {
		return sendBufferByteArrayOutputStream;
	}


	@Override
	public DataOutputStream getSendBufferDataOutputStream() {
		return sendBufferDataOutputStream;
	}


	@Override
	public void setLastMesageReceivedID(byte lastMessageReceivedID) {
		this.lastMessageReceivedID = lastMessageReceivedID;
	}


	@Override
	public void setMessageSentID(byte messageSentID) {
		this.messageSentID = messageSentID;	
	}


	@Override
	public void setSendBufferByteArrayOutputStream(
			ByteArrayOutputStream sendBufferByteArrayOutputStream) {
		this.sendBufferByteArrayOutputStream = sendBufferByteArrayOutputStream;
	}


	@Override
	public void setSendBufferDataOutputStream(
			DataOutputStream sendBufferDataOutputStream) {
		this.sendBufferDataOutputStream = sendBufferDataOutputStream;
	}

}