package org.ugasp.impl.logging.log4j;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;


/**
 * This class provides an implementation of org.ugasp.logging.Logger with the log4j framework.
 * It provides two logs methods:
 * <ul>
 * <li>one used when the log message is set by the developer</li>
 * <li>another one used when the developer wants to insert both
 * its message and the stack trace of an exception caught in the log</li>
 * </ul>
 * Level of logs used by both of those methods are the one defined with the Logger interface.
 */
public class LoggerImpl implements org.ugasp.logging.Logger{

	//iPojo injection field
	private String log4jConfigurationFile;
	
	public LoggerImpl(){} 

	/**
	 * Method used to initialize the log4j environment with the log4j configuration XML file.
	 * This method is called by iPOJO during the validate lifecycle callback.
	 */
	public void init(){
		File configurationFile = new File(log4jConfigurationFile);

		String configFileEntirePath;
		try {
			configFileEntirePath = configurationFile.getCanonicalPath();
		} catch (IOException e) {
			// No need to detail error message, since log4j does it automatically
			return;
		}
		// load configuration file
		DOMConfigurator.configureAndWatch(configFileEntirePath);
		log(INFO, this.getClass(),"uGASP Log4j Logging Manager instantiated.");
	}


	/**
	 * Log a message for a special class with an associated level.
	 * 
	 * @param logLevel	the level of the log, that must be chosen between TRACE, DEBUG, INFO, WAR, ERROR or FATAL
	 * @param clazz		the class where the log is located
	 * @param log		the log message
	 */
	public void log(byte logLevel, Class<?> clazz, String log){
		// retrieve the specific logger matching with a class: with log4j, each class matches with one and only Logger.
		Logger logger = Logger.getLogger(clazz.getName());
		switch(logLevel){
		case FATAL: 
			logger.fatal(log);
			break;
		case ERROR:
			logger.error(log);
			break;
		case WARN:
			logger.warn(log);
			break;
		case INFO:
			logger.info(log);
			break;
		case DEBUG:
			logger.debug(log);
			break;
		case TRACE:
			logger.trace(log);
			break;
		}
	}
	
	
	/**
	 * Log a message for a special class with an associated level. 
	 * In addition to the other log method, this one has a Throwable parameter. The stack trace of this Throwable
	 * is provided by the log.
	 * Note that this method should be use for a level of Logger superior or equals to 
	 * 
	 * @param logLevel	the level of the log, that should be chosen between WARN, ERROR or FATAL
	 * @param clazz		the class where the log is located
	 * @param log		the log message
	 * @param t			the Throwable whose stack trace is logged
	 */
	public void log(byte logLevel, Class<?> clazz, String log, Throwable t){
		// retrieve the specific logger matching with a class: with log4j, each class matches with one and only Logger.
		Logger logger = Logger.getLogger(clazz.getName());
		switch(logLevel){
		case FATAL: 
			logger.fatal(log,t);
			break;
		case ERROR:
			logger.error(log,t);
			break;
		case WARN:
			logger.warn(log,t);
			break;
		case INFO:
			logger.info(log,t);
			break;
		case DEBUG:
			logger.debug(log,t);
			break;
		case TRACE:
			logger.trace(log,t);
			break;
		}
	}
}
