/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Emilio Boudoni
 */
package org.ugasp.impl.service.external.player;


import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map.Entry;
import java.util.Set;
import java.util.Vector;

import org.ugasp.core.Actor;
import org.ugasp.core.ActorSession;
import org.ugasp.core.ApplicationInstance;
import org.ugasp.core.MasterApplicationInstance;
import org.ugasp.core.Platform;
import org.ugasp.core.Session;
import org.ugasp.core.utils.ApplicationInstanceInfos;
import org.ugasp.core.utils.ApplicationInstanceInfos.Player;

import org.ugasp.protocol.moods.CustomTypes;

import org.ugasp.eventmodel.event.*;
import org.ugasp.eventmodel.event.impl.*;

import org.ugasp.service.external.lobby.Lobby;
import org.ugasp.service.external.player.PlayerConnector;

import org.ugasp.logging.Logger;


public class PlayerConnectorImpl implements PlayerConnector{
    
	//iPojo dynamic binding var
	private Platform core;
	private Lobby lobby;
	
	public PlayerConnectorImpl(){
		core.log(Logger.INFO, this.getClass(), "Player Connector instantiated");
	}
	
	
	@Override
	public short create(short sessionID, int minActors, int maxActors, String[] actors) {
		if (minActors > 0 && minActors<= maxActors){
			//log(Logger.DEBUG,"Create application instance request from sessionID = "+sID+" with minActors = "+minActors+" and maxActors = "+maxActors);
			core.log(Logger.DEBUG,this.getClass(),"Create Application Instance request from sessionID = "+sessionID+" with minActors = "+minActors+" and maxActors = "+maxActors);
			// create an application instance owned by the sid represented actor
			// with minActors and maxActors 
			// if the table actors is empty then the application instance is public, else it is private
			short actorSessionID = core.createApplicationInstance(Integer.parseInt(sessionID+""),minActors,maxActors,actors);
			core.log(Logger.DEBUG,this.getClass(),"Create Application Instance response to sessionID = "+sessionID+": actorSessionID = "+actorSessionID);
			return actorSessionID;
		}
		else{
			core.log(Logger.DEBUG,this.getClass(),"Invalid minActors and maxActors parameters in create application instance request ("+minActors+","+maxActors+")");
		    return 0;
		}
	}


	@Override
	public void end(short actorSessionID) {
		EndEvent ee = new EndEventImpl(actorSessionID);
		ApplicationInstance appIns = core.getApplicationInstanceFromActorSessionID(actorSessionID);
		if(appIns != null){
			core.log(Logger.DEBUG,this.getClass(),"End application instance resquest from actorSessionID = "+actorSessionID+" to end application instance with ID = "+appIns.getApplicationInstanceID());
			appIns.onEndEvent(ee);
		}else{
			core.log(Logger.DEBUG,this.getClass(),"End application instance resquest from actorSessionID = "+actorSessionID+" failed since no matching application instance could be found.");
		}
	}


	@Override
	/**
	 * Registration of the user with his choice of username and password
	 * @return 	- short actorID, permanent actorID associated with the userID and applicationID
	 * 			- 0 if the user have not the appropriate rights 
	 */
	public int firstLogin(short userID, short appID, String username, String password) {
		core.log(Logger.DEBUG,this.getClass(),"First login request from userID = "+userID+" with username = "+username+" for application with ID = "+appID);
		int actorID = core.getDBManager().registerNewActor(userID,appID,username,password);
		core.log(Logger.DEBUG,this.getClass(),"First login response to userID = "+userID+": actorID = "+actorID);
		return actorID;
	}
	
	
	@Override
	public int login(short actorID, String username, String password) {
		// login, return a short representing the sessionID 
		// or the short "0" if the authentication informations are not valid.
		core.log(Logger.DEBUG,this.getClass(),"Login request from actorID = "+actorID+" with username = "+username);
		int sessionID = core.login(actorID, username, password);
		core.log(Logger.DEBUG,this.getClass(),"Login response to actorID = "+actorID+": sessionID = "+sessionID);
		return sessionID;
	}

	
	@Override
	public void remoteMethodAsynchronousInvocation(short senderActorSessionID, short recipientActorSessionID, String invokedMethodXMLSerialized) {
		try{
			// take the platform, the AI , the MAI and the Session associated to ASID
			ApplicationInstance appIns = core.getApplicationInstanceFromActorSessionID(senderActorSessionID);
			//log(Logger.DEBUG,"RMAI received from actorSessionID = "+senderActorSessionID+" designed for actorSessionID = "+recipientActorSessionID+")");
			core.log(Logger.DEBUG,this.getClass(),"RMAI received from actorSessionID = "+senderActorSessionID+" designed for actorSessionID = "+recipientActorSessionID+")");
			RMAIEvent rmaiEvent = new RMAIEventImpl(senderActorSessionID,recipientActorSessionID,invokedMethodXMLSerialized);
			// raise event
			appIns.onRMAIEvent(rmaiEvent);
		}
		catch(Throwable t){
			//log(Logger.ERROR,e.getMessage());
			core.log(Logger.DEBUG,this.getClass(),"Exception on remote Method Asynchronous Invocation",t);
		}
	}
	
	@Override
	public void sendMooDSObject(DataInputStream dis, short actorSessionID) {
		try{
			// take the platform, the AI , the MAI and the Session associated to ASID
			ApplicationInstance appIns = core.getApplicationInstanceFromActorSessionID(actorSessionID);
			CustomTypes customTypes = appIns.getCustomTypes();
	        Hashtable h = customTypes.decodeData(dis);
	        if (h.size()!=0){
	        	core.log(Logger.DEBUG,this.getClass(),"MooDS data received from actorSessionID = "+actorSessionID+", "+h.size()+" objects are present in the map");
	        	MooDSEvent me = new MooDSEventImpl(actorSessionID,h);
	            appIns.onMooDSEvent(me);
	        }
		}
		catch(Throwable t){
			core.log(Logger.DEBUG,this.getClass(),"Exception on sending of MooDS object",t);
		}
	}
	
	
	@Override
	public short joinAI(short sessionID, short applicationInstanceID) {
		short actorSessionID;
		
		if (applicationInstanceID == 0){ 
			//Join Randomly an Application Instance
			//log(Logger.DEBUG,"Join random application instance request from sessionID = "+sID);
			core.log(Logger.DEBUG,this.getClass(),"Join Random Application Instance request from sessionID = "+sessionID);
			actorSessionID = (short) core.joinRandomApplicationInstance(sessionID);
		}
		else{
			// Join the specific Application Instance	
			//log(Logger.DEBUG,"Join request from sessionID = "+sID+" to join applicationInstanceID = "+aIID);
			core.log(Logger.DEBUG,this.getClass(),"Join Application Instance request from sessionID = "+sessionID+" to join applicationInstanceID = "+applicationInstanceID);
			actorSessionID = (short) core.joinApplicationInstance(sessionID,applicationInstanceID);
		}
		core.log(Logger.DEBUG,this.getClass(),"Join Application Instance response to sessionID = "+sessionID+": actorSessionID = "+actorSessionID);
		return actorSessionID;
	}
	

	@Override
	public void lobby(short sID, DataOutputStream dos) {
		try{
			//log(Logger.DEBUG,"Lobby request from sessionID = "+sID);
			core.log(Logger.DEBUG,this.getClass(),"Lobby request from sessionID = "+sID);
			Vector<ApplicationInstanceInfos> lobbyInfos = null;
			// get a lock on the lobby
			synchronized (this) {
				// requiring the lobby infos to the platform for the application associated with the sID
				lobbyInfos = lobby.getApplicationInstancesInfos((int) sID);
			}
						
			// push in the DataOutputStream the number of application instances
			dos.writeShort(lobbyInfos.size());
			
			for (Iterator<ApplicationInstanceInfos> iter = lobbyInfos.iterator(); iter.hasNext();){
				ApplicationInstanceInfos appInstanceInfos = iter.next();
				
				// push the aIID of the current application instance
				dos.writeShort(appInstanceInfos.getApplicationInstanceID());
				
				// push the minimum starting number of actors
				dos.writeShort(appInstanceInfos.getMinActors());
				
				// push the number of actors currently connected in
				dos.writeShort(appInstanceInfos.getNbPlayers());
				
				// push the maximum number of actors 
				dos.writeShort(appInstanceInfos.getMaxActors());
				
				for (Iterator<Player> iterPlayer= appInstanceInfos.getPlayerList().iterator(); iterPlayer.hasNext();){
					Player player = iterPlayer.next();
					
					// push the pseudo and the actor ID of the current actor
					dos.writeUTF(player.getPseudoName());
					dos.writeInt(player.getActorId());
				}
				
				// retrieve optional infos entries
				Set<Entry<String, String>> optionalInfos = appInstanceInfos.getOptionalInfos().entrySet();
				// push the number of data in the optional info map
				dos.writeShort(optionalInfos.size());
				// write each entry in the set
				for (Entry<String, String> entry : optionalInfos) {
					dos.writeUTF(entry.getKey());
					dos.writeUTF(entry.getValue());
				}
			}
		}
		catch (Throwable t){
			core.log(Logger.ERROR, this.getClass(), "Exception on the retrieving of lobby", t);
		}
	}


	@Override
	public String name(short actorSessionID, String name) {
		core.log(Logger.DEBUG,this.getClass(),"Renaming request from actorSessionID = "+actorSessionID+", new desired name = "+name);
		ApplicationInstance appIns = core.getApplicationInstanceFromActorSessionID(actorSessionID);
		ActorSession actorSession = appIns.getActorSession(actorSessionID);
		MasterApplicationInstance masterApp = core.getMasterApplicationInstanceFromSessionID(actorSession.getSessionID());
		Actor actor = masterApp.getActor(actorSession.getActorID());

		// treat the new pseudoname
		actorSession.setPseudoName(appIns.treatPseudo(name));
		actor.setPseudoName(actorSession.getPseudoName());
		core.getDBManager().saveLastUsedPseudo(actor.getActorID(),
				actor.getPseudoName());

		// raise the new pseudo to all other players
		appIns.raisePseudoModification(actorSession.getActorSessionID(), actorSession.getPseudoName());
		if (!name.equals(actorSession.getPseudoName())){
			actorSession.getPseudoName();
		}
		core.log(Logger.DEBUG,this.getClass(),"Renaming response to actorSessionID = "+actorSessionID+", new pseudo = "+actorSession.getPseudoName());
		return actorSession.getPseudoName();
	}


	@Override
	public void quit(int sessionID) {
		core.log(Logger.DEBUG,this.getClass(),"Quit platform request from sessionID = "+sessionID);
		core.quit(sessionID);
	}


	@Override
	public void quitAI(short actorSessionID) {
		// create quit event
		QuitEvent qe = new QuitEventImpl(actorSessionID);
		//log(Logger.DEBUG,"Quit Application Instance request from actosSessionID = "+aSID);
		core.log(Logger.DEBUG,this.getClass(),"Quit Application Instance request from actosSessionID = "+actorSessionID);
		// take the platform, the AI , the MAI and the Session associated to ASID
		ApplicationInstance appIns = core.getApplicationInstanceFromActorSessionID(actorSessionID);
		if(appIns != null){
			ActorSession aS = appIns.getActorSession(actorSessionID);
			int sID = aS.getSessionID();
			// raise event to AI
			appIns.onQuitEvent(qe);
			MasterApplicationInstance masterApp = core.getMasterApplicationInstanceFromSessionID(sID);
			Session session = masterApp.getSession(sID);
			// TODO what ??? to tell it is invalid ??
			session.setActorSessionID((short)0);
		}else{
			core.log(Logger.WARN, this.getClass(), "Quit Application Instance request failed since no Application Instance matching with actor session ID = "+actorSessionID+" could be retrieved.");
		}
		
	}


	@Override
	public boolean start(short actorSessionID) {
		StartEvent se = new StartEventImpl(actorSessionID);
		core.log(Logger.DEBUG,this.getClass(),"Start Application Instance request from actorSessionID = "+actorSessionID);
		ApplicationInstance appIns = core.getApplicationInstanceFromActorSessionID(actorSessionID);
		boolean isStarted = false;
		if ((appIns.getOwnerActorSessionID()==actorSessionID) && appIns.isStartable()){
			appIns.onStartEvent(se);
			isStarted = true;
		}
		core.log(Logger.DEBUG,this.getClass(),"Start Application Instance response to actorSessionID = "+actorSessionID+", : "+isStarted);
		return isStarted;
	}
	
	@Override
    public String lobby(short sessionID) {
            return null;
    }
	
	/*********************************************************************************************************/
	/********************************  			Management of Events			******************************/
	/*************************************************** *****************************************************/

	@Override
	public void getEvents(DataOutputStream dos, short actorSessionID) {
		try{
			core.log(Logger.TRACE,this.getClass(),"Get events for actor session with ID = "+actorSessionID);
			ApplicationInstance applicationInstance = core.getApplicationInstanceFromActorSessionID(actorSessionID);
			ActorSession actorSession = applicationInstance.getActorSession(actorSessionID);
			// get events, and purge them from the actorSession
			encodeEvents(actorSession.getEvents(),dos, applicationInstance);
		    // destroy the actor session linked to actorSessionID
		    // and if all actor sessions are destroyed calling the owner master app
		    // to destroy the application instance.
		    if (applicationInstance.isDestroyable()){
		    	QuitEventImpl qe = new QuitEventImpl(actorSessionID);
	            applicationInstance.onQuitEvent(qe);  
	            if (applicationInstance.getActorSessionsList().isEmpty()){
	            	int sessionID = actorSession.getSessionID();
	    			MasterApplicationInstance masterApp = core.getMasterApplicationInstanceFromSessionID(sessionID);
	    			masterApp.removeApplicationInstance(applicationInstance.getApplicationInstanceID());
	            }    
	        }
		}catch(Throwable t){
			core.log(Logger.DEBUG,this.getClass(),"Exception on the getting of application instance events",t);
		}
	}
	
	
	private void encodeEvents(Vector events,DataOutputStream dos, ApplicationInstance instance) throws Exception {
		for(Iterator iter = events.iterator(); iter.hasNext();){
			Event e = (Event) iter.next();
			// retrieve event code
			byte code = e.getCode();
			switch (code){
			case EventsType.JOIN_EVENT_TYPE:
				JoinEvent je = (JoinEvent) e;
				encodeJoinEvent(je, dos);
				break;
			case EventsType.START_EVENT_TYPE:
				StartEvent se = (StartEvent) e;
				encodeStartEvent(se, dos);
				break;
			case EventsType.END_EVENT_TYPE:
				EndEvent ee = (EndEvent) e;
				encodeEndEvent(ee, dos);
				break;
			case EventsType.QUIT_EVENT_TYPE:
				QuitEvent qe = (QuitEvent) e;
				encodeQuitEvent(qe, dos);				
				break;
			case EventsType.RMAI_EVENT_TYPE:
				RMAIEvent rmaiEvent = (RMAIEvent) e;
				encodeRMAIEvent(rmaiEvent, dos);	
				break;
			case EventsType.MooDS_EVENT_TYPE:
				MooDSEvent moodsEvent = (MooDSEvent)e;
				encodeMooDSEvent(moodsEvent, dos, instance);				
				break;
			default: break;
			}
		}
	}
	
	
	private void encodeJoinEvent(JoinEvent joinEvent, DataOutputStream dos) throws IOException{
		core.log(Logger.DEBUG, this.getClass(),"Start to encode JoinEvent("+joinEvent.getActorSessionID()+","+joinEvent.getUsername()+")");
		dos.writeByte(joinEvent.getCode());
		dos.writeShort(joinEvent.getActorSessionID());
		dos.writeUTF(joinEvent.getUsername());
	}
	
	private void encodeStartEvent(StartEvent startEvent, DataOutputStream dos) throws IOException{
		core.log(Logger.DEBUG, this.getClass(),"Start to encode StartEvent("+startEvent.getActorSessionID()+")");
		dos.writeByte(startEvent.getCode());
		dos.writeShort(startEvent.getActorSessionID());
	}
	
	private void encodeEndEvent(EndEvent endEvent, DataOutputStream dos) throws IOException{
		core.log(Logger.DEBUG, this.getClass(), "Start to encode EndEvent("+endEvent.getActorSessionID()+")");
		dos.writeByte(endEvent.getCode());
		dos.writeShort(endEvent.getActorSessionID());
	}
	
	private void encodeQuitEvent(QuitEvent quitEvent, DataOutputStream dos) throws IOException{
		core.log(Logger.DEBUG, this.getClass(), "Start to encode QuitEvent("+quitEvent.getActorSessionID()+")");
		dos.writeByte(quitEvent.getCode());
		dos.writeShort(quitEvent.getActorSessionID());
	}
	
	private void encodeRMAIEvent(RMAIEvent rmaiEvent, DataOutputStream dos) throws IOException{
		core.log(Logger.DEBUG, this.getClass(), "Start to encode RMAIEvent("+rmaiEvent.getActorSessionID()+","+rmaiEvent.getRecipientActorSessionID());
		byte[] rmaiEncoded = getEncodedRMAIEvent(rmaiEvent);
		if(rmaiEncoded != null){
			dos.writeByte(rmaiEvent.getCode());
			dos.write(rmaiEncoded);
		}
	}
	
	private void encodeMooDSEvent(MooDSEvent moodsEvent, DataOutputStream dos, ApplicationInstance instance) throws Exception{
		core.log(Logger.DEBUG, this.getClass(),"Start to encode MooDSEvent("+moodsEvent.getActorSessionID()+") containing "+moodsEvent.getMooDSData().size()+" objects");
		dos.writeByte(moodsEvent.getCode());
		// sender of the event
		dos.writeShort(moodsEvent.getActorSessionID());
		// retrieve customTypes matching with application instance
		CustomTypes customTypes = instance.getCustomTypes();            
		//encoding MooDS data on the dos
		customTypes.encodeData(moodsEvent.getMooDSData(),dos);
	}
	
	/*
	 * Return a byte array representing the RMAIEvent.
	 * 
	 * @param 	rmaiEvent the RMAIEvent to encode
	 * @return	a byte[] containing several attributes of an RMAIEvent,
	 * 			null if encoding has failed.
	 */
	private byte[] getEncodedRMAIEvent(RMAIEvent rmaiEvent){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		try {
			// sender of the event
			dos.writeShort(rmaiEvent.getActorSessionID());
			// suitable recipient for this event
			dos.writeShort(rmaiEvent.getRecipientActorSessionID());
			// string representation of the remote method invoked on the recipient
			dos.writeUTF(rmaiEvent.getInvokedMethodXMLSerialized());
			// return the bytes encoded
			return baos.toByteArray();
		} catch (Throwable t) {
			core.log(Logger.WARN, this.getClass(), "Exception on encoding of RMAI: it won't be sent.",t);
		}
		return null;
	}
}

