/***
 * uGASP: an OSGi middleware dedicated to ubiquitous multiplayer gaming.
 * Copyright (C) 2008 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.ugasp.gamelogic;


import java.util.Map;

import org.ugasp.eventmodel.listener.*;
import org.ugasp.core.ApplicationInstance;
import org.ugasp.protocol.moods.CustomTypes;


/**
 * A game logic running on uGASP must implement this interface.
 * 
 * @author PELLERIN Romain (Ubidreams, romain.pellerin@ubidreams.com )
 * @author ADGEG Gabriel (Telecom Sud Paris, gabriel.adgeg@telecom-sudparis.eu)
 */
public interface GameLogic extends  OnEndEvent, OnJoinEvent, OnQuitEvent, OnStartEvent, OnRMAIEvent, OnMooDSEvent{
	
    /**
     * Link the application instance to the game server logic.
     * 
     * @param applicationInstance
     */
    public void setOwnerAI(ApplicationInstance applicationInstance);
    
    
    /**
     * Retrieve the actor ID linked to the actor session ID.
     * (persistency) Provide the storage of player game informations. 
     * 
     * @param actorSessionID
     * @param dataEvent the DataEvent to send
     */
    public int retrieveActorID(short actorSessionID);
    
    
    /**
     * Retrieve the application ID.
     * (persistency) Provide the storage of player game informations. 
     */
    public int retrieveApplicationID();
    
    
    /**
	 * Return the pseudo of a user from its actorSessionId.
	 * 
	 * @return	the pseudo of user (Beware, pseudo is NOT the user name).
	 */
    public String getPseudoByActorSessionId(short actorSessionId);
    
    
    /**
	 * Return actorSessionId of a user from its pseudo.
	 * 
	 * @return	the actorSessionId of user
	 * 			-1 if no actorSessionId is matching with pseudo.
	 */
    public short getActorSessionIdByPseudo(String pseudo);
    

    /**
     * Get a map of optional in game informations. This informations can be retrieved in ApplicationInstanceInfos object 
     * within the Lobby phase.
     * By example, this map should be used in some game to add data such as mission level matching with one instance.
     * In this case, an entry of map should be :
     * key = "level" - value = "Mission to Mars"
     * 
     * @return 	the Map containing every in game optional information
     */
    public Map<String,String> getOptionalInstanceInfos();
    
    /**
     * Retrieve the CustomTypes encoder/decoder.
     */
	public CustomTypes getCustomTypes();
 
	
	/**
	 * Send a MooDS Message object (i.e. an object generated by MooDS) from a client to another one.
	 * 
	 * @param o the object to send
	 * @param senderActorSessionID the actorSessionID of the sender
	 * @param destinationActorSessionID the actorSessionID of the recipient
	 */
	public void sendMooDSMessageToPlayer(Object moodsObject, short senderActorSessionID, short destinationActorSessionID);

		
    /**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP server (RMAIClientManager.ActorSessionID.uGASP_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP server (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 * @throws IOException
	 */
    public void remoteMethodAsynchronousInvocation(short recipientGASPClientActorSessionID, String methodName);

	
    /**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP server (RMAIClientManager.ActorSessionID.uGASP_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP server (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 	
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 * @throws IOException
	 */
    public void remoteMethodAsynchronousInvocation(short recipientGASPClientActorSessionID, String methodName, Object param0);
	
	
    /**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP server (RMAIClientManager.ActorSessionID.uGASP_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP server (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 	
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @param param1					second parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 * @throws IOException
	 */
    public void remoteMethodAsynchronousInvocation(short recipientGASPClientActorSessionID, String methodName, Object param0,Object param1);
	
	
    /**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP server (RMAIClientManager.ActorSessionID.uGASP_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP server (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 	
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param params					parameters of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 * @throws IOException
	 */
    public void remoteMethodAsynchronousInvocation(short recipientGASPClientActorSessionID, String methodName, Object param0,Object param1, Object param2);
	
	
    /**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP server (RMAIClientManager.ActorSessionID.uGASP_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP server (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 	
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param params					parameters of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 * @throws IOException
	 */
    public void remoteMethodAsynchronousInvocation(short recipientGASPClientActorSessionID, String methodName, Object param0, Object param1, Object param2, Object param3);
	
	
    /**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP server (RMAIClientManager.ActorSessionID.uGASP_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP server (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 	
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param params					parameters of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 * @throws IOException
	 */
    public void remoteMethodAsynchronousInvocation(short recipientGASPClientActorSessionID, String methodName, Object param0, Object param1, Object param2, Object param3, Object param4);
	
	
    /**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP server (RMAIClientManager.ActorSessionID.uGASP_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP server (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 	
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param params					parameters of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 * @throws IOException
	 */
    public void remoteMethodAsynchronousInvocation(short recipientGASPClientActorSessionID, String methodName, Object param0, Object param1, Object param2, Object param3, Object param4, Object param5);
	
	
    /**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP server (RMAIClientManager.ActorSessionID.uGASP_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP server (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_SERVER.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 	
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param params					parameters of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 * @throws IOException
	 */
    public void remoteMethodAsynchronousInvocation(short recipientGASPClientActorSessionID, String methodName, Object param0, Object param1, Object param2, Object param3, Object param4, Object param5, Object param6);
}
